@@ -4,14 +4,8 @@
 
 /* Obtain additive fe for ub data; assume r=0, without covar */
 // [[Rcpp::export]]
-List fe_ad_iter(arma::mat Y,
-                arma::mat Y0,
-                arma::mat I,
-                arma::mat W,
-                int force,
-                double tolerate,
-                int max_iter = 500)
-{
+List fe_ad_iter(const arma::mat& Y, const arma::mat& Y0, const arma::mat& I, const arma::mat& W, int force,
+                double tolerate, int max_iter = 500) {
 
   int T = Y.n_rows;
   int N = Y.n_cols;
@@ -20,12 +14,9 @@ List fe_ad_iter(arma::mat Y,
   int niter = 0;
   int use_weight;
 
-  if (Y.n_rows == W.n_rows && Y.n_cols == W.n_cols)
-  {
+  if (Y.n_rows == W.n_rows && Y.n_cols == W.n_cols) {
     use_weight = 1;
-  }
-  else
-  {
+  } else {
     use_weight = 0;
   }
 
@@ -43,26 +34,20 @@ List fe_ad_iter(arma::mat Y,
   List Y_ad;
   List Y_fe_ad;
 
-  while (dif > tolerate && niter <= 500)
-  {
-    if (use_weight == 1)
-    {
+  while (dif > tolerate && niter <= 500) {
+    if (use_weight == 1) {
       YY = wE_adj(Y, fit, W, I); // e step: expeactation
-    }
-    else
-    {
+    } else {
       YY = E_adj(Y, fit, I);
     }
 
     Y_ad = Y_demean(YY, force);
 
     mu_Y = as<double>(Y_ad["mu_Y"]);
-    if (force == 1 || force == 3)
-    {
+    if (force == 1 || force == 3) {
       alpha_Y = as<arma::mat>(Y_ad["alpha_Y"]);
     }
-    if (force == 2 || force == 3)
-    {
+    if (force == 2 || force == 3) {
       xi_Y = as<arma::mat>(Y_ad["xi_Y"]);
     }
 
@@ -70,12 +55,10 @@ List fe_ad_iter(arma::mat Y,
 
     fit = as<arma::mat>(Y_fe_ad["FE_ad"]);
 
-    if (use_weight == 1)
-    {
-      dif = arma::norm(W % (fit - fit_old), "fro") / arma::norm(W % (fit_old), "fro");
-    }
-    else
-    {
+    if (use_weight == 1) {
+      dif = arma::norm(W % (fit - fit_old), "fro") /
+            arma::norm(W % (fit_old), "fro");
+    } else {
       dif = arma::norm(fit - fit_old, "fro") / arma::norm(fit_old, "fro");
     }
 
@@ -92,13 +75,11 @@ List fe_ad_iter(arma::mat Y,
   result["fit"] = fit;
   result["niter"] = niter;
   result["e"] = e;
-  if (force == 1 || force == 3)
-  {
+  if (force == 1 || force == 3) {
     alpha = as<arma::mat>(Y_fe_ad["alpha"]);
     result["alpha"] = alpha;
   }
-  if (force == 2 || force == 3)
-  {
+  if (force == 2 || force == 3) {
     xi = as<arma::mat>(Y_fe_ad["xi"]);
     result["xi"] = xi;
   }
@@ -107,17 +88,9 @@ List fe_ad_iter(arma::mat Y,
 
 /* Obtain additive fe for ub data; assume r=0, with covariates */
 // [[Rcpp::export]]
-List fe_ad_covar_iter(arma::cube XX,
-                      arma::mat xxinv,
-                      arma::mat Y,
-                      arma::mat Y0,
-                      arma::mat I,
-                      arma::mat beta0,
-                      arma::mat W,
-                      int force,
-                      double tolerate,
-                      int max_iter = 500)
-{
+List fe_ad_covar_iter(const arma::cube& XX, const arma::mat& xxinv, const arma::mat& Y, const arma::mat& Y0,
+                      const arma::mat& I, const arma::mat& beta0, const arma::mat& W, int force,
+                      double tolerate, int max_iter = 500) {
   int T = Y.n_rows;
   int N = Y.n_cols;
   int p = XX.n_slices;
@@ -125,18 +98,14 @@ List fe_ad_covar_iter(arma::cube XX,
   int niter = 0;
   int use_weight;
 
-  if (Y.n_rows == W.n_rows && Y.n_cols == W.n_cols)
-  {
+  if (Y.n_rows == W.n_rows && Y.n_cols == W.n_cols) {
     use_weight = 1;
-  }
-  else
-  {
+  } else {
     use_weight = 0;
   }
 
   arma::mat beta(p, 1, arma::fill::zeros);
-  if (beta0.n_rows == p)
-  {
+  if (beta0.n_rows == p) {
     beta = beta0;
   }
 
@@ -152,8 +121,7 @@ List fe_ad_covar_iter(arma::cube XX,
 
   List ife_inner;
   arma::mat covar_fit(T, N, arma::fill::zeros);
-  for (int i = 0; i < p; i++)
-  {
+  for (int i = 0; i < p; i++) {
     covar_fit = covar_fit + XX.slice(i) * beta(i);
   }
 
@@ -161,33 +129,25 @@ List fe_ad_covar_iter(arma::cube XX,
   fit_old = fit;
   arma::mat FE = fit - covar_fit; // initial fixed effects
 
-  while (dif > tolerate && niter <= max_iter)
-  {
+  while (dif > tolerate && niter <= max_iter) {
 
     YY = E_adj(Y, fit, I);
     // m1: estimate beta
-    if (use_weight == 1)
-    {
+    if (use_weight == 1) {
       beta = wpanel_beta(XX, xxinv, W, YY, FE); //  xxinv is xwxinv
-    }
-    else
-    {
+    } else {
       beta = panel_beta(XX, xxinv, YY, FE);
     }
 
     covar_fit.zeros();
-    for (int i = 0; i < p; i++)
-    {
+    for (int i = 0; i < p; i++) {
       covar_fit = covar_fit + XX.slice(i) * beta(i);
     }
 
     // m2: estimate interactive fe, additive fe, and mu
-    if (use_weight == 1)
-    {
+    if (use_weight == 1) {
       U = wE_adj(YY - covar_fit, FE, W, I); // e step: expeactation
-    }
-    else
-    {
+    } else {
       U = E_adj(YY - covar_fit, FE, I);
     }
 
@@ -196,12 +156,10 @@ List fe_ad_covar_iter(arma::cube XX,
     FE = as<arma::mat>(ife_inner["FE"]); // overall fe
     fit = covar_fit + FE;
 
-    if (use_weight == 1)
-    {
-      dif = arma::norm(W % (fit - fit_old), "fro") / arma::norm(W % (fit_old), "fro");
-    }
-    else
-    {
+    if (use_weight == 1) {
+      dif = arma::norm(W % (fit - fit_old), "fro") /
+            arma::norm(W % (fit_old), "fro");
+    } else {
       dif = arma::norm(fit - fit_old, "fro") / arma::norm(fit_old, "fro");
     }
 
@@ -217,17 +175,14 @@ List fe_ad_covar_iter(arma::cube XX,
   result["fit"] = fit;
   result["niter"] = niter;
   result["e"] = e;
-  if (p > 0)
-  {
+  if (p > 0) {
     result["beta"] = beta;
   }
-  if (force == 1 || force == 3)
-  {
+  if (force == 1 || force == 3) {
     alpha = as<arma::mat>(ife_inner["alpha"]);
     result["alpha"] = alpha;
   }
-  if (force == 2 || force == 3)
-  {
+  if (force == 2 || force == 3) {
     xi = as<arma::mat>(ife_inner["xi"]);
     result["xi"] = xi;
   }
@@ -236,18 +191,11 @@ List fe_ad_covar_iter(arma::cube XX,
 
 /* Obtain additive fe for ub data; assume r>0 but p=0*/
 // [[Rcpp::export]]
-List fe_ad_inter_iter(arma::mat Y,
-                      arma::mat Y0,
-                      arma::mat I,
-                      arma::mat W,
+List fe_ad_inter_iter(const arma::mat& Y, const arma::mat& Y0, const arma::mat& I, const arma::mat& W,
                       int force,
                       int mc, // whether pca or mc method
-                      int r,
-                      int hard,
-                      double lambda,
-                      double tolerate,
-                      int max_iter = 1000)
-{
+                      int r, int hard, double lambda, double tolerate,
+                      int max_iter = 1000) {
   int T = Y.n_rows;
   int N = Y.n_cols;
   double mu = 0;
@@ -257,21 +205,15 @@ List fe_ad_inter_iter(arma::mat Y,
   int use_weight;
   int r_burnin;
   int d;
-  if (T <= N)
-  {
+  if (T <= N) {
     d = T;
-  }
-  else
-  {
+  } else {
     d = N;
   }
 
-  if (Y.n_rows == W.n_rows && Y.n_cols == W.n_cols)
-  {
+  if (Y.n_rows == W.n_rows && Y.n_cols == W.n_cols) {
     use_weight = 1;
-  }
-  else
-  {
+  } else {
     use_weight = 0;
   }
 
@@ -296,52 +238,39 @@ List fe_ad_inter_iter(arma::mat Y,
   fit = Y0;
   fit_old = fit;
   int stop_burnin = 0;
-  while (dif > tolerate && niter <= max_iter)
-  {
-    if (use_weight == 1)
-    {
+  while (dif > tolerate && niter <= max_iter) {
+    if (use_weight == 1) {
       YY = wE_adj(Y, fit, W, I); // e step: expectation
-    }
-    else
-    {
+    } else {
       YY = E_adj(Y, fit, I);
     }
-    if (mc == 0)
-    {
-      if (use_weight == 1 && stop_burnin == 0)
-      {
+    if (mc == 0) {
+      if (use_weight == 1 && stop_burnin == 0) {
         r_burnin = d - niter;
-        if (r_burnin <= r)
-        {
+        if (r_burnin <= r) {
           r_burnin = r;
         }
         ife_inner = ife(YY, force, 0, r_burnin, 0, 0);
-      }
-      else
-      {
+      } else {
         ife_inner = ife(YY, force, 0, r, 0, 0);
       }
-    }
-    else
-    {
+    } else {
       ife_inner = ife(YY, force, 1, 1, hard, lambda);
     }
     fit = as<arma::mat>(ife_inner["FE"]); // new overall fe
 
-    if (use_weight == 1)
-    {
-      dif = arma::norm(W % (fit - fit_old), "fro") / arma::norm(W % (fit_old), "fro");
-    }
-    else
-    {
+    if (use_weight == 1) {
+      dif = arma::norm(W % (fit - fit_old), "fro") /
+            arma::norm(W % (fit_old), "fro");
+    } else {
       dif = arma::norm(fit - fit_old, "fro") / arma::norm(fit_old, "fro");
     }
 
     fit_old = fit;
     niter = niter + 1;
 
-    if (dif <= tolerate && niter <= d && use_weight == 1 && stop_burnin == 0 && mc == 0)
-    {
+    if (dif <= tolerate && niter <= d && use_weight == 1 && stop_burnin == 0 &&
+        mc == 0) {
       stop_burnin = 1;
       dif = 1.0;
       niter = 0;
@@ -351,8 +280,7 @@ List fe_ad_inter_iter(arma::mat Y,
   }
   e = FE_adj(YY - fit, I);
   FE_inter_use = as<arma::mat>(ife_inner["FE_inter_use"]);
-  if (arma::accu(abs(FE_inter_use)) < 1e-10)
-  {
+  if (arma::accu(abs(FE_inter_use)) < 1e-10) {
     validF = 0;
   }
 
@@ -364,24 +292,20 @@ List fe_ad_inter_iter(arma::mat Y,
   result["e"] = e;
   result["validF"] = validF;
 
-  if (force == 1 || force == 3)
-  {
+  if (force == 1 || force == 3) {
     alpha = as<arma::mat>(ife_inner["alpha"]);
     result["alpha"] = alpha;
   }
-  if (force == 2 || force == 3)
-  {
+  if (force == 2 || force == 3) {
     xi = as<arma::mat>(ife_inner["xi"]);
     result["xi"] = xi;
   }
-  if (mc == 0)
-  {
+  if (mc == 0) {
     result["lambda"] = as<arma::mat>(ife_inner["lambda"]);
     result["factor"] = as<arma::mat>(ife_inner["factor"]);
     result["VNT"] = as<arma::mat>(ife_inner["VNT"]);
   }
-  if (use_weight == 1 && mc == 0)
-  {
+  if (use_weight == 1 && mc == 0) {
     result["burn_in"] = abs(1 - stop_burnin);
   }
   return (result);
@@ -389,21 +313,12 @@ List fe_ad_inter_iter(arma::mat Y,
 
 /* Obtain additive fe for ub data; assume r>0 p>0*/
 // [[Rcpp::export]]
-List fe_ad_inter_covar_iter(arma::cube XX,
-                            arma::mat xxinv,
-                            arma::mat Y,
-                            arma::mat Y0,
-                            arma::mat I,
-                            arma::mat W,
-                            arma::mat beta0,
-                            int force,
+List fe_ad_inter_covar_iter(const arma::cube& XX, const arma::mat& xxinv, const arma::mat& Y,
+                            const arma::mat& Y0, const arma::mat& I, const arma::mat& W,
+                            const arma::mat& beta0, int force,
                             int mc, // whether pca or mc method
-                            int r,
-                            int hard,
-                            double lambda,
-                            double tolerate,
-                            int max_iter = 1000)
-{
+                            int r, int hard, double lambda, double tolerate,
+                            int max_iter = 1000) {
   int T = Y.n_rows;
   int N = Y.n_cols;
   int p = XX.n_slices;
@@ -413,27 +328,20 @@ List fe_ad_inter_covar_iter(arma::cube XX,
   int use_weight;
   int r_burnin;
   int d;
-  if (T <= N)
-  {
+  if (T <= N) {
     d = T;
-  }
-  else
-  {
+  } else {
     d = N;
   }
 
-  if (Y.n_rows == W.n_rows && Y.n_cols == W.n_cols)
-  {
+  if (Y.n_rows == W.n_rows && Y.n_cols == W.n_cols) {
     use_weight = 1;
-  }
-  else
-  {
+  } else {
     use_weight = 0;
   }
 
   arma::mat beta(p, 1, arma::fill::zeros);
-  if (beta0.n_rows == p)
-  {
+  if (beta0.n_rows == p) {
     beta = beta0;
   }
   double mu = 0;
@@ -451,7 +359,9 @@ List fe_ad_inter_covar_iter(arma::cube XX,
   List pf;
   arma::mat F;
   arma::mat L;
-
+  // arma::mat fit_FE_inter_use_old(T, N, arma::fill::ones);
+  // arma::mat fit_FE_inter_use(T, N, arma::fill::zeros);
+  // double dif_FE_inter_use = 1.0;
   // initial value for ife
 
   // if (hard == 0) {
@@ -473,75 +383,62 @@ List fe_ad_inter_covar_iter(arma::cube XX,
   //}
 
   arma::mat covar_fit(T, N, arma::fill::zeros);
-  for (int i = 0; i < p; i++)
-  {
+  for (int i = 0; i < p; i++) {
     covar_fit = covar_fit + XX.slice(i) * beta(i);
   }
   arma::mat FE = fit - covar_fit;
 
   int stop_burnin = 0;
-  while (dif > tolerate && niter <= max_iter)
-  {
+  // while ((dif > tolerate || dif_FE_inter_use > tolerate )&& niter <= max_iter) {
+  while (dif > tolerate && niter <= max_iter) {
 
     YY = E_adj(Y, fit, I); // e-step: expectation
 
     // m1: estimate beta
-    if (use_weight == 1)
-    {
+    if (use_weight == 1) {
       beta = wpanel_beta(XX, xxinv, W, YY, FE); //  xxinv is xwxinv
-    }
-    else
-    {
+    } else {
       beta = panel_beta(XX, xxinv, YY, FE);
     }
 
     covar_fit.zeros();
-    for (int i = 0; i < p; i++)
-    {
+    for (int i = 0; i < p; i++) {
       covar_fit = covar_fit + XX.slice(i) * beta(i);
     }
 
     // m2: estimate interactive fe, additive fe, and mu
-    if (use_weight == 1)
-    {
+    if (use_weight == 1) {
       U = wE_adj(YY - covar_fit, FE, W, I); // e step: expeactation
-    }
-    else
-    {
+    } else {
       U = E_adj(YY - covar_fit, FE, I);
     }
 
-    if (use_weight == 1 && stop_burnin == 0)
-    {
+    if (use_weight == 1 && stop_burnin == 0) {
       r_burnin = d - niter;
-      if (r_burnin <= r)
-      {
+      if (r_burnin <= r) {
         r_burnin = r;
       }
       ife_inner = ife(U, force, mc, r_burnin, hard, lambda);
-    }
-    else
-    {
+    } else {
       ife_inner = ife(U, force, mc, r, hard, lambda);
     }
 
     FE = as<arma::mat>(ife_inner["FE"]);
     fit = covar_fit + FE; // overall fe */
-
-    if (use_weight == 1)
-    {
-      dif = arma::norm(W % (fit - fit_old), "fro") / arma::norm(W % (fit_old), "fro");
-    }
-    else
-    {
+    // fit_FE_inter_use = as<arma::mat>(ife_inner["FE_inter_use"]);
+    if (use_weight == 1) {
+      dif = arma::norm(W % (fit - fit_old), "fro") /
+            arma::norm(W % (fit_old), "fro");
+    } else {
       dif = arma::norm(fit - fit_old, "fro") / arma::norm(fit_old, "fro");
     }
-
+    // dif_FE_inter_use = arma::norm(fit_FE_inter_use - fit_FE_inter_use_old, "fro") / arma::norm(fit_FE_inter_use_old, "fro");
+    // fit_FE_inter_use_old = fit_FE_inter_use;
     fit_old = fit;
     niter = niter + 1;
 
-    if (dif <= tolerate && niter <= d && use_weight == 1 && stop_burnin == 0 && mc == 0)
-    {
+    if (dif <= tolerate && niter <= d && use_weight == 1 && stop_burnin == 0 &&
+        mc == 0) {
       stop_burnin = 1;
       dif = 1.0;
       niter = 0;
@@ -552,8 +449,7 @@ List fe_ad_inter_covar_iter(arma::cube XX,
   e = FE_adj(Y - fit, I);
 
   FE_inter_use = as<arma::mat>(ife_inner["FE_inter_use"]);
-  if (arma::accu(abs(FE_inter_use)) < 1e-10)
-  {
+  if (arma::accu(abs(FE_inter_use)) < 1e-10) {
     validF = 0;
   }
   List result;
@@ -565,18 +461,15 @@ List fe_ad_inter_covar_iter(arma::cube XX,
   result["fit"] = fit;
   result["validF"] = validF;
 
-  if (force == 1 || force == 3)
-  {
+  if (force == 1 || force == 3) {
     alpha = as<arma::mat>(ife_inner["alpha"]);
     result["alpha"] = alpha;
   }
-  if (force == 2 || force == 3)
-  {
+  if (force == 2 || force == 3) {
     xi = as<arma::mat>(ife_inner["xi"]);
     result["xi"] = xi;
   }
-  if (mc == 0)
-  {
+  if (mc == 0) {
     L = as<arma::mat>(ife_inner["lambda"]);
     F = as<arma::mat>(ife_inner["factor"]);
     VNT = as<arma::mat>(ife_inner["VNT"]);
@@ -584,8 +477,7 @@ List fe_ad_inter_covar_iter(arma::cube XX,
     result["factor"] = F;
     result["VNT"] = VNT;
   }
-  if (use_weight == 1 && mc == 0)
-  {
+  if (use_weight == 1 && mc == 0) {
     result["burn_in"] = abs(1 - stop_burnin);
   }
   return (result);
@@ -593,14 +485,8 @@ List fe_ad_inter_covar_iter(arma::cube XX,
 
 /* Main iteration for beta */
 // [[Rcpp::export]]
-List beta_iter(arma::cube X,
-               arma::mat xxinv,
-               arma::mat Y,
-               int r,
-               double tolerate,
-               arma::mat beta0,
-               int max_iter)
-{
+List beta_iter(const arma::cube& X, const arma::mat& xxinv, const arma::mat& Y, int r,
+               double tolerate, const arma::mat& beta0, int max_iter) {
 
   /* beta.new: computed beta under iteration with error precision=tolerate
    factor: estimated factor
@@ -614,8 +500,7 @@ List beta_iter(arma::cube X,
   int b_r = beta0.n_rows;
   double beta_norm = 1.0;
   arma::mat beta(p, 1, arma::fill::zeros);
-  if (b_r == p)
-  {
+  if (b_r == p) {
     beta = beta0;
   } // beta should have the same dimension as X, if not it will be reset to 0
   arma::mat beta_old = beta;
@@ -624,8 +509,7 @@ List beta_iter(arma::cube X,
 
   /* starting value */
   arma::mat U = Y;
-  for (int k = 0; k < p; k++)
-  {
+  for (int k = 0; k < p; k++) {
     U = U - X.slice(k) * beta(k);
   }
   List pf = panel_factor(U, r);
@@ -634,16 +518,14 @@ List beta_iter(arma::cube X,
 
   /* Loop */
   int niter = 0;
-  while ((beta_norm > tolerate) && (niter < max_iter))
-  {
+  while ((beta_norm > tolerate) && (niter < max_iter)) {
     niter++;
     FE = F * L.t();
     beta = panel_beta(X, xxinv, Y, FE);
     beta_norm = arma::norm(beta - beta_old, "fro");
     beta_old = beta;
     U = Y;
-    for (int k = 0; k < p; k++)
-    {
+    for (int k = 0; k < p; k++) {
       U = U - X.slice(k) * beta(k);
     }
     pf = panel_factor(U, r);
